////////////////////////////////////////////////////////////////////////////////
//  Project: Pong Game Clone
//  Author: Vladeta Stojanovic (vladeta_stojanovic@yahoo.co.uk)
//  File: core.cpp
//  Description: All the game core functions go here, which includes all of the
//  image laoding, collision detection, keypad input and gba hardware operation function
//  implementations.
//  Version: v0.1
//  
//  Date (last revision): March 16, 2009
//  Date (creation): March 30, 2009
//
//
//  Revision history notes:
//  ---Date---|---Revision---|---Revisor---|
//
//  Credits:  Portions of this code were based on the examples form the book
//            "Programming The Nintendo Game Boy Advance: The Unofficial Guide"
//            by Jonathan S. Harbour (http://www.jharbour.com).
//            Paddle collision detection code based on the Pong game code
//            by Aaron Cox (http://www.aaroncox.net).
//            Additional code and examples are derived from the code presented
//            in TONC v1.4 by J Vijn (http://www.coranac.com/projects/#tonc).
//
//  License: You are allowed to use the code in this file for presonal,
//           scholarship and/or non-commercial purpuses. This program code
//           comes with NO WARRANTY.
//
///////////////////////////////////////////////////////////////////////////////

#include "defs.hpp"
#include "core.hpp"

//create an array of 128 sprites equal to OAM
Sprite sprites[128];

//create arry of 128 sprite handler attribute specific sprites
Sprite_handler game_sprites[128];

//create an array ofb utton key press values
bool buttons[10];

//vsync function implementation
void vsync()
{
    while(REG_VCOUNT >= 160){}; //do nothing till vertical draw updates
    while(REG_VCOUNT < 160){}; //do nothing till vertical blank updates
}

//sprite handler initialization function implementation
void init_sprites(int num, bool paddle, int x, int y, int size, int color, int tile_index)
{
    unsigned int sprite_size = 0;
    
    //init the main sprite attributes
    game_sprites[num].alive = 1;
    game_sprites[num].x = x;
    game_sprites[num].y = y;
    game_sprites[num].size = size;
    
    //in modes 0-2, sprite memory starts at 0
    sprites[num].attribute2 = tile_index;
    
    //check if the sprite is the paddle
    if(paddle == true)
    {
         sprites[num].attribute0 = color | WIDE | y; // 1 << 9 | y
         game_sprites[num].dir_y = PADDLE_Y_SPEED;
    }else
    {
    
    //initialize the sprite
    sprites[num].attribute0 = color | y; // 1 << 9 | y
    game_sprites[num].dir_y = BALL_X_Y_SPEED;
    game_sprites[num].dir_x = BALL_X_Y_SPEED;
    }
    //select the size for the sprite
    switch(size)
    {
        case 8: {sprite_size = SIZE_8; break;}
        case 16: {sprite_size = SIZE_16; break;}
        case 32: {sprite_size = SIZE_32; break;}
        case 64: {sprite_size = SIZE_64; break;}
        default: {sprite_size = SIZE_16; break;}
    }
    
    sprites[num].attribute1 = sprite_size | x; //1 << 9 | x
}

//hide_sprites() function implementation
void hide_sprites()
{
   for (int i = 0; i < 128; ++i)
   {
       sprites[i].attribute0 = 160; //y
       sprites[i].attribute1 = 240; //x
   }
}

//move_sprites() function implementation
//todo: revise further
void move_sprite(int num)
{
    //u16 x_bit = 65024; //0xFE00
    //u16 y_bit = 65280; //0xFF00
    
    //clear old x coord values
    sprites[num].attribute1 = sprites[num].attribute1 & 0xFE00;
    //set new x coord value
    sprites[num].attribute1 = sprites[num].attribute1 | game_sprites[num].x;
    
    //clear old y coord values
    sprites[num].attribute0 = sprites[num].attribute0 & 0xFF00;
    //set new y coord value
    sprites[num].attribute0 = sprites[num].attribute0 | game_sprites[num].y;
}

// copy the sprite array into OAM memory
void update_oam(void)
{
    int n;
    unsigned short* temp;
    temp = (unsigned short*)sprites;

    for(n = 0; n < 128*4; n++)
        SpriteMem[n] = temp[n];
}

//timer function implementation
void timer(int time)
{
    for (int i = 0; i <= time; ++i)
    {
        //do nothing
    }
}

//check buttons functions implementation
void check_buttons()
{
    //store the status of each button in an array
    buttons[0] = !((*BUTTONS) & BUTTON_A);
    buttons[1] = !((*BUTTONS) & BUTTON_B);
    buttons[2] = !((*BUTTONS) & BUTTON_LEFT);
    buttons[3] = !((*BUTTONS) & BUTTON_RIGHT);
    buttons[4] = !((*BUTTONS) & BUTTON_UP);
    buttons[5] = !((*BUTTONS) & BUTTON_DOWN);
    buttons[6] = !((*BUTTONS) & BUTTON_START);
    buttons[7] = !((*BUTTONS) & BUTTON_SELECT);
    buttons[8] = !((*BUTTONS) & BUTTON_L);
    buttons[9] = !((*BUTTONS) & BUTTON_R);
}

//press button function implementation
//returns the status of the button pressed, either 1 or 0
bool button_pressed(int button)
{
    switch(button)
    {
        case BUTTON_A: return buttons[0];
        case BUTTON_B: return buttons[1];
        case BUTTON_LEFT: return buttons[2];
        case BUTTON_RIGHT: return buttons[3];
        case BUTTON_UP: return buttons[4];
        case BUTTON_DOWN: return buttons[5];
        case BUTTON_START: return buttons[6];
        case BUTTON_SELECT: return buttons[7];
        case BUTTON_L: return buttons[8];
        case BUTTON_R: return buttons[9];
    }
    
    return false;
}

